@@ -2,28 +2,31 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2010 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
-** Commercial Usage
-**
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
 **
 ** GNU Lesser General Public License Usage
 **
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this file.
+** Please review the following information to ensure the GNU Lesser General
+** Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** If you are unsure which license is appropriate for your use, please
-** contact the sales department at http://qt.nokia.com/contact.
+** In addition, as a special exception, Nokia gives you certain additional
+** rights. These rights are described in the Nokia Qt LGPL Exception
+** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+**
+** Other Usage
+**
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+** If you have questions regarding the use of this file, please contact
+** Nokia at qt-info@nokia.com.
 **
 **************************************************************************/
 
@@ -34,37 +37,43 @@
 #include "cppeditorenums.h"
 #include "cppfilewizard.h"
 #include "cpphoverhandler.h"
-#include "cppquickfix.h"
+#include "cppoutline.h"
+#include "cpptypehierarchy.h"
+#include "cppsnippetprovider.h"
+#include "cppquickfixassistant.h"
 
 #include <coreplugin/icore.h>
 #include <coreplugin/coreconstants.h>
 #include <coreplugin/mimedatabase.h>
-#include <coreplugin/uniqueidmanager.h>
 #include <coreplugin/fileiconprovider.h>
 #include <coreplugin/actionmanager/actionmanager.h>
 #include <coreplugin/actionmanager/actioncontainer.h>
 #include <coreplugin/actionmanager/command.h>
+#include <coreplugin/id.h>
 #include <coreplugin/editormanager/editormanager.h>
 #include <coreplugin/progressmanager/progressmanager.h>
-#include <texteditor/completionsupport.h>
-#include <texteditor/fontsettings.h>
-#include <texteditor/storagesettings.h>
+#include <coreplugin/navigationwidget.h>
 #include <texteditor/texteditoractionhandler.h>
 #include <texteditor/texteditorplugin.h>
 #include <texteditor/texteditorsettings.h>
 #include <texteditor/texteditorconstants.h>
+#include <cplusplus/ModelManagerInterface.h>
 #include <cpptools/cpptoolsconstants.h>
-#include <cpptools/cppmodelmanagerinterface.h>
+#include <cpptools/cpptoolssettings.h>
 
-#include <QtCore/QFileInfo>
-#include <QtCore/QSettings>
-#include <QtCore/QTimer>
-#include <QtCore/QCoreApplication>
+#include <QFileInfo>
+#include <QSettings>
+#include <QTimer>
+#include <QCoreApplication>
+#include <QStringList>
 
-#include <QtGui/QMenu>
+#include <QMenu>
 
+using namespace CppEditor;
 using namespace CppEditor::Internal;
 
+void registerQuickFixes(ExtensionSystem::IPlugin *plugIn);
+
 enum { QUICKFIX_INTERVAL = 20 };
 
 //////////////////////////// CppEditorFactory /////////////////////////////
@@ -77,21 +86,21 @@ CppEditorFactory::CppEditorFactory(CppPlugin *owner) :
             << QLatin1String(CppEditor::Constants::CPP_SOURCE_MIMETYPE)
             << QLatin1String(CppEditor::Constants::CPP_HEADER_MIMETYPE);
 
-#if !defined(Q_WS_MAC) && !defined(Q_WS_WIN)
+#if !defined(Q_OS_MAC) && !defined(Q_WS_WIN)
     Core::FileIconProvider *iconProvider = Core::FileIconProvider::instance();
-    Core::MimeDatabase *mimeDatabase = Core::ICore::instance()->mimeDatabase();
-    iconProvider->registerIconOverlayForMimeType(QIcon(":/cppeditor/images/qt_cpp.png"),
+    Core::MimeDatabase *mimeDatabase = Core::ICore::mimeDatabase();
+    iconProvider->registerIconOverlayForMimeType(QIcon(QLatin1String(":/cppeditor/images/qt_cpp.png")),
                                                  mimeDatabase->findByType(QLatin1String(CppEditor::Constants::CPP_SOURCE_MIMETYPE)));
-    iconProvider->registerIconOverlayForMimeType(QIcon(":/cppeditor/images/qt_c.png"),
+    iconProvider->registerIconOverlayForMimeType(QIcon(QLatin1String(":/cppeditor/images/qt_c.png")),
                                                  mimeDatabase->findByType(QLatin1String(CppEditor::Constants::C_SOURCE_MIMETYPE)));
-    iconProvider->registerIconOverlayForMimeType(QIcon(":/cppeditor/images/qt_h.png"),
+    iconProvider->registerIconOverlayForMimeType(QIcon(QLatin1String(":/cppeditor/images/qt_h.png")),
                                                  mimeDatabase->findByType(QLatin1String(CppEditor::Constants::CPP_HEADER_MIMETYPE)));
 #endif
 }
 
-QString CppEditorFactory::id() const
+Core::Id CppEditorFactory::id() const
 {
-    return QLatin1String(CppEditor::Constants::CPPEDITOR_ID);
+    return CppEditor::Constants::CPPEDITOR_ID;
 }
 
 QString CppEditorFactory::displayName() const
@@ -99,18 +108,18 @@ QString CppEditorFactory::displayName() const
     return tr(CppEditor::Constants::CPPEDITOR_DISPLAY_NAME);
 }
 
-Core::IFile *CppEditorFactory::open(const QString &fileName)
+Core::IDocument *CppEditorFactory::open(const QString &fileName)
 {
     Core::IEditor *iface = Core::EditorManager::instance()->openEditor(fileName, id());
-    return iface ? iface->file() : 0;
+    return iface ? iface->document() : 0;
 }
 
 Core::IEditor *CppEditorFactory::createEditor(QWidget *parent)
 {
-    CPPEditor *editor = new CPPEditor(parent);
+    CPPEditorWidget *editor = new CPPEditorWidget(parent);
     editor->setRevisionsVisible(true);
     m_owner->initializeEditor(editor);
-    return editor->editableInterface();
+    return editor->editor();
 }
 
 QStringList CppEditorFactory::mimeTypes() const
@@ -121,33 +130,28 @@ QStringList CppEditorFactory::mimeTypes() const
 ///////////////////////////////// CppPlugin //////////////////////////////////
 
 static inline
-        Core::Command *createSeparator(Core::ActionManager *am,
-                                       QObject *parent,
-                                       const QList<int> &context,
-                                       const char *id)
+Core::Command *createSeparator(Core::ActionManager *am,
+                               QObject *parent,
+                               Core::Context &context,
+                               const char *id)
 {
     QAction *separator = new QAction(parent);
     separator->setSeparator(true);
-    return am->registerAction(separator, QLatin1String(id), context);
+    return am->registerAction(separator, Core::Id(id), context);
 }
 
 CppPlugin *CppPlugin::m_instance = 0;
 
 CppPlugin::CppPlugin() :
     m_actionHandler(0),
-    m_sortedMethodOverview(false),
+    m_sortedOutline(false),
     m_renameSymbolUnderCursorAction(0),
     m_findUsagesAction(0),
-    m_updateCodeModelAction(0)
-
+    m_updateCodeModelAction(0),
+    m_openTypeHierarchyAction(0),
+    m_quickFixProvider(0)
 {
     m_instance = this;
-
-    m_quickFixCollector = 0;
-    m_quickFixTimer = new QTimer(this);
-    m_quickFixTimer->setInterval(20);
-    m_quickFixTimer->setSingleShot(true);
-    connect(m_quickFixTimer, SIGNAL(timeout()), this, SLOT(quickFixNow()));
 }
 
 CppPlugin::~CppPlugin()
@@ -161,52 +165,50 @@ CppPlugin *CppPlugin::instance()
     return m_instance;
 }
 
-void CppPlugin::initializeEditor(CPPEditor *editor)
+void CppPlugin::initializeEditor(CPPEditorWidget *editor)
 {
     m_actionHandler->setupActions(editor);
 
+    editor->setLanguageSettingsId(QLatin1String(CppTools::Constants::CPP_SETTINGS_ID));
     TextEditor::TextEditorSettings::instance()->initializeEditor(editor);
 
-    // auto completion
-    connect(editor, SIGNAL(requestAutoCompletion(TextEditor::ITextEditable*, bool)),
-            TextEditor::Internal::CompletionSupport::instance(), SLOT(autoComplete(TextEditor::ITextEditable*, bool)));
-
-    // quick fix
-    connect(editor, SIGNAL(requestQuickFix(TextEditor::ITextEditable*)),
-            this, SLOT(quickFix(TextEditor::ITextEditable*)));
-
     // method combo box sorting
-    connect(this, SIGNAL(methodOverviewSortingChanged(bool)),
-            editor, SLOT(setSortedMethodOverview(bool)));
+    connect(this, SIGNAL(outlineSortingChanged(bool)),
+            editor, SLOT(setSortedOutline(bool)));
 }
 
-void CppPlugin::setSortedMethodOverview(bool sorted)
+void CppPlugin::setSortedOutline(bool sorted)
 {
-    m_sortedMethodOverview = sorted;
-    emit methodOverviewSortingChanged(sorted);
+    m_sortedOutline = sorted;
+    emit outlineSortingChanged(sorted);
 }
 
-bool CppPlugin::sortedMethodOverview() const
+bool CppPlugin::sortedOutline() const
 {
-    return m_sortedMethodOverview;
+    return m_sortedOutline;
 }
 
-CPPQuickFixCollector *CppPlugin::quickFixCollector() const
-{ return m_quickFixCollector; }
+CppQuickFixAssistProvider *CppPlugin::quickFixProvider() const
+{
+    return m_quickFixProvider;
+}
 
 bool CppPlugin::initialize(const QStringList & /*arguments*/, QString *errorMessage)
 {
-    Core::ICore *core = Core::ICore::instance();
-
-    if (!core->mimeDatabase()->addMimeTypes(QLatin1String(":/cppeditor/CppEditor.mimetypes.xml"), errorMessage))
+    if (!Core::ICore::mimeDatabase()->addMimeTypes(QLatin1String(":/cppeditor/CppEditor.mimetypes.xml"), errorMessage))
         return false;
 
     addAutoReleasedObject(new CppEditorFactory(this));
     addAutoReleasedObject(new CppHoverHandler);
+    addAutoReleasedObject(new CppOutlineWidgetFactory);
+    addAutoReleasedObject(new CppTypeHierarchyFactory);
+    addAutoReleasedObject(new CppSnippetProvider);
 
-    m_quickFixCollector = new CPPQuickFixCollector;
-    addAutoReleasedObject(m_quickFixCollector);
+    m_quickFixProvider = new CppQuickFixAssistProvider;
+    addAutoReleasedObject(m_quickFixProvider);
+    registerQuickFixes(this);
 
+    QObject *core = Core::ICore::instance();
     CppFileWizard::BaseFileWizardParameters wizardParameters(Core::IWizard::FileWizard);
 
     wizardParameters.setCategory(QLatin1String(Constants::WIZARD_CATEGORY));
@@ -229,18 +231,20 @@ bool CppPlugin::initialize(const QStringList & /*arguments*/, QString *errorMess
     wizardParameters.setId(QLatin1String("C.Header"));
     addAutoReleasedObject(new CppFileWizard(wizardParameters, Header, core));
 
-    QList<int> context;
-    context << core->uniqueIDManager()->uniqueIdentifier(CppEditor::Constants::C_CPPEDITOR);
+    Core::Context context(CppEditor::Constants::C_CPPEDITOR);
 
-    Core::ActionManager *am = core->actionManager();
+    Core::ActionManager *am = Core::ICore::actionManager();
     Core::ActionContainer *contextMenu= am->createMenu(CppEditor::Constants::M_CONTEXT);
 
     Core::Command *cmd;
-    Core::ActionContainer *cppToolsMenu = am->actionContainer(QLatin1String(CppTools::Constants::M_TOOLS_CPP));
+    Core::ActionContainer *cppToolsMenu = am->actionContainer(Core::Id(CppTools::Constants::M_TOOLS_CPP));
+
+    cmd = am->command(Core::Id(CppTools::Constants::SWITCH_HEADER_SOURCE));
+    contextMenu->addAction(cmd);
 
     QAction *jumpToDefinition = new QAction(tr("Follow Symbol Under Cursor"), this);
     cmd = am->registerAction(jumpToDefinition,
-        Constants::JUMP_TO_DEFINITION, context);
+        Constants::JUMP_TO_DEFINITION, context, true);
     cmd->setDefaultKeySequence(QKeySequence(Qt::Key_F2));
     connect(jumpToDefinition, SIGNAL(triggered()),
             this, SLOT(jumpToDefinition()));
@@ -249,8 +253,8 @@ bool CppPlugin::initialize(const QStringList & /*arguments*/, QString *errorMess
 
     QAction *switchDeclarationDefinition = new QAction(tr("Switch Between Method Declaration/Definition"), this);
     cmd = am->registerAction(switchDeclarationDefinition,
-        Constants::SWITCH_DECLARATION_DEFINITION, context);
-    cmd->setDefaultKeySequence(QKeySequence("Shift+F2"));
+        Constants::SWITCH_DECLARATION_DEFINITION, context, true);
+    cmd->setDefaultKeySequence(QKeySequence(tr("Shift+F2")));
     connect(switchDeclarationDefinition, SIGNAL(triggered()),
             this, SLOT(switchDeclarationDefinition()));
     contextMenu->addAction(cmd);
@@ -263,21 +267,37 @@ bool CppPlugin::initialize(const QStringList & /*arguments*/, QString *errorMess
     contextMenu->addAction(cmd);
     cppToolsMenu->addAction(cmd);
 
-    m_renameSymbolUnderCursorAction = new QAction(tr("Rename Symbol Under Cursor"), this);
-    cmd = am->registerAction(m_renameSymbolUnderCursorAction,
-        Constants::RENAME_SYMBOL_UNDER_CURSOR, context);
-    cmd->setDefaultKeySequence(QKeySequence("CTRL+SHIFT+R"));
-    connect(m_renameSymbolUnderCursorAction, SIGNAL(triggered()), this, SLOT(renameSymbolUnderCursor()));
+    m_openTypeHierarchyAction = new QAction(tr("Open Type Hierarchy"), this);
+    cmd = am->registerAction(m_openTypeHierarchyAction, Constants::OPEN_TYPE_HIERARCHY, context);
+    cmd->setDefaultKeySequence(QKeySequence(tr("Ctrl+Shift+T")));
+    connect(m_openTypeHierarchyAction, SIGNAL(triggered()), this, SLOT(openTypeHierarchy()));
     contextMenu->addAction(cmd);
     cppToolsMenu->addAction(cmd);
 
+    // Refactoring sub-menu
+    Core::Context globalContext(Core::Constants::C_GLOBAL);
+    Core::Command *sep = createSeparator(am, this, globalContext,
+                                         Constants::SEPARATOR2);
+    sep->action()->setObjectName(QLatin1String(Constants::M_REFACTORING_MENU_INSERTION_POINT));
+    contextMenu->addAction(sep);
+    contextMenu->addAction(createSeparator(am, this, globalContext,
+                                           Constants::SEPARATOR3));
+
+    m_renameSymbolUnderCursorAction = new QAction(tr("Rename Symbol Under Cursor"),
+                                                  this);
+    cmd = am->registerAction(m_renameSymbolUnderCursorAction,
+                             Constants::RENAME_SYMBOL_UNDER_CURSOR,
+                             context);
+    cmd->setDefaultKeySequence(QKeySequence(tr("CTRL+SHIFT+R")));
+    connect(m_renameSymbolUnderCursorAction, SIGNAL(triggered()),
+            this, SLOT(renameSymbolUnderCursor()));
+    cppToolsMenu->addAction(cmd);
+
     // Update context in global context
-    QList<int> globalContext;
-    globalContext.append(Core::Constants::C_GLOBAL_ID);
-    cppToolsMenu->addAction(createSeparator(am, this, globalContext, CppEditor::Constants::SEPARATOR2));
+    cppToolsMenu->addAction(createSeparator(am, this, globalContext, CppEditor::Constants::SEPARATOR4));
     m_updateCodeModelAction = new QAction(tr("Update Code Model"), this);
-    cmd = am->registerAction(m_updateCodeModelAction, QLatin1String(Constants::UPDATE_CODEMODEL), globalContext);
-    CppTools::CppModelManagerInterface *cppModelManager = CppTools::CppModelManagerInterface::instance();
+    cmd = am->registerAction(m_updateCodeModelAction, Core::Id(Constants::UPDATE_CODEMODEL), globalContext);
+    CPlusPlus::CppModelManagerInterface *cppModelManager = CPlusPlus::CppModelManagerInterface::instance();
     connect(m_updateCodeModelAction, SIGNAL(triggered()), cppModelManager, SLOT(updateModifiedSourceFiles()));
     cppToolsMenu->addAction(cmd);
 
@@ -296,37 +316,41 @@ bool CppPlugin::initialize(const QStringList & /*arguments*/, QString *errorMess
     cmd = am->command(TextEditor::Constants::UN_COMMENT_SELECTION);
     contextMenu->addAction(cmd);
 
-    connect(core->progressManager(), SIGNAL(taskStarted(QString)),
+    connect(Core::ICore::progressManager(), SIGNAL(taskStarted(QString)),
             this, SLOT(onTaskStarted(QString)));
-    connect(core->progressManager(), SIGNAL(allTasksFinished(QString)),
+    connect(Core::ICore::progressManager(), SIGNAL(allTasksFinished(QString)),
             this, SLOT(onAllTasksFinished(QString)));
+
+    connect(Core::ICore::editorManager(), SIGNAL(currentEditorChanged(Core::IEditor*)), SLOT(currentEditorChanged(Core::IEditor*)));
+
     readSettings();
     return true;
 }
 
 void CppPlugin::readSettings()
 {
-    m_sortedMethodOverview = Core::ICore::instance()->settings()->value("CppTools/SortedMethodOverview", false).toBool();
+    m_sortedOutline = Core::ICore::settings()->value(QLatin1String("CppTools/SortedMethodOverview"), false).toBool();
 }
 
 void CppPlugin::writeSettings()
 {
-    Core::ICore::instance()->settings()->setValue("CppTools/SortedMethodOverview", m_sortedMethodOverview);
+    Core::ICore::settings()->setValue(QLatin1String("CppTools/SortedMethodOverview"), m_sortedOutline);
 }
 
 void CppPlugin::extensionsInitialized()
 {
 }
 
-void CppPlugin::aboutToShutdown()
+ExtensionSystem::IPlugin::ShutdownFlag CppPlugin::aboutToShutdown()
 {
     writeSettings();
+    return SynchronousShutdown;
 }
 
 void CppPlugin::switchDeclarationDefinition()
 {
     Core::EditorManager *em = Core::EditorManager::instance();
-    CPPEditor *editor = qobject_cast<CPPEditor*>(em->currentEditor()->widget());
+    CPPEditorWidget *editor = qobject_cast<CPPEditorWidget*>(em->currentEditor()->widget());
     if (editor)
         editor->switchDeclarationDefinition();
 }
@@ -334,7 +358,7 @@ void CppPlugin::switchDeclarationDefinition()
 void CppPlugin::jumpToDefinition()
 {
     Core::EditorManager *em = Core::EditorManager::instance();
-    CPPEditor *editor = qobject_cast<CPPEditor*>(em->currentEditor()->widget());
+    CPPEditorWidget *editor = qobject_cast<CPPEditorWidget*>(em->currentEditor()->widget());
     if (editor)
         editor->jumpToDefinition();
 }
@@ -342,7 +366,7 @@ void CppPlugin::jumpToDefinition()
 void CppPlugin::renameSymbolUnderCursor()
 {
     Core::EditorManager *em = Core::EditorManager::instance();
-    CPPEditor *editor = qobject_cast<CPPEditor*>(em->currentEditor()->widget());
+    CPPEditorWidget *editor = qobject_cast<CPPEditorWidget*>(em->currentEditor()->widget());
     if (editor)
         editor->renameSymbolUnderCursor();
 }
@@ -350,50 +374,49 @@ void CppPlugin::renameSymbolUnderCursor()
 void CppPlugin::findUsages()
 {
     Core::EditorManager *em = Core::EditorManager::instance();
-    CPPEditor *editor = qobject_cast<CPPEditor*>(em->currentEditor()->widget());
+    CPPEditorWidget *editor = qobject_cast<CPPEditorWidget*>(em->currentEditor()->widget());
     if (editor)
         editor->findUsages();
 }
 
-void CppPlugin::quickFix(TextEditor::ITextEditable *editable)
-{
-    m_currentTextEditable = editable;
-    quickFixNow();
-}
-
-void CppPlugin::quickFixNow()
-{
-    if (! m_currentTextEditable)
-        return;
-
-    Core::EditorManager *em = Core::EditorManager::instance();
-    CPPEditor *currentEditor = qobject_cast<CPPEditor*>(em->currentEditor()->widget());
-
-    if (CPPEditor *editor = qobject_cast<CPPEditor*>(m_currentTextEditable->widget())) {
-        if (currentEditor == editor) {
-            if (editor->isOutdated())
-                m_quickFixTimer->start(QUICKFIX_INTERVAL);
-            else
-                TextEditor::Internal::CompletionSupport::instance()->quickFix(m_currentTextEditable);
-        }
-    }
-}
-
 void CppPlugin::onTaskStarted(const QString &type)
 {
-    if (type == CppTools::Constants::TASK_INDEX) {
+    if (type == QLatin1String(CppTools::Constants::TASK_INDEX)) {
         m_renameSymbolUnderCursorAction->setEnabled(false);
         m_findUsagesAction->setEnabled(false);
         m_updateCodeModelAction->setEnabled(false);
+        m_openTypeHierarchyAction->setEnabled(false);
     }
 }
 
 void CppPlugin::onAllTasksFinished(const QString &type)
 {
-    if (type == CppTools::Constants::TASK_INDEX) {
+    if (type == QLatin1String(CppTools::Constants::TASK_INDEX)) {
         m_renameSymbolUnderCursorAction->setEnabled(true);
         m_findUsagesAction->setEnabled(true);
         m_updateCodeModelAction->setEnabled(true);
+        m_openTypeHierarchyAction->setEnabled(true);
+    }
+}
+
+void CppPlugin::currentEditorChanged(Core::IEditor *editor)
+{
+    if (! editor)
+        return;
+
+    else if (CPPEditorWidget *textEditor = qobject_cast<CPPEditorWidget *>(editor->widget())) {
+        textEditor->semanticRehighlight(/*force = */ true);
+    }
+}
+
+void CppPlugin::openTypeHierarchy()
+{
+    Core::EditorManager *em = Core::EditorManager::instance();
+    CPPEditorWidget *editor = qobject_cast<CPPEditorWidget*>(em->currentEditor()->widget());
+    if (editor) {
+        Core::NavigationWidget *navigation = Core::NavigationWidget::instance();
+        navigation->activateSubWidget(Core::Id(Constants::TYPE_HIERARCHY_ID));
+        emit typeHierarchyRequested();
     }
 }
 
