@@ -2,7 +2,7 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -35,11 +35,12 @@
 
 #include <coreplugin/icore.h>
 
-#include <QtCore/QSettings>
-#include <QtGui/QMainWindow>
-#include <QtGui/QStringListModel>
-#include <QtGui/QCompleter>
-#include <QtGui/QKeyEvent>
+#include <QSettings>
+#include <QMainWindow>
+#include <QStringListModel>
+#include <QCompleter>
+#include <QKeyEvent>
+#include <QScrollArea>
 
 using namespace Find;
 using namespace Find::Internal;
@@ -174,9 +175,9 @@ void FindToolWindow::setCurrentFilter(int index)
         if (i == index) {
             m_configWidget = configWidget;
             if (m_currentFilter)
-                disconnect(m_currentFilter, SIGNAL(changed()), this, SLOT(updateButtonStates()));
+                disconnect(m_currentFilter, SIGNAL(enabledChanged(bool)), this, SLOT(updateButtonStates()));
             m_currentFilter = m_filters.at(i);
-            connect(m_currentFilter, SIGNAL(changed()), this, SLOT(updateButtonStates()));
+            connect(m_currentFilter, SIGNAL(enabledChanged(bool)), this, SLOT(updateButtonStates()));
             updateButtonStates();
             if (m_configWidget) {
                 m_ui.configWidget->layout()->addWidget(m_configWidget);
@@ -186,6 +187,19 @@ void FindToolWindow::setCurrentFilter(int index)
                 configWidget->setParent(0);
         }
     }
+    QWidget *w = m_ui.configWidget;
+    while (w) {
+        QScrollArea *sa = qobject_cast<QScrollArea *>(w);
+        if (sa) {
+            sa->updateGeometry();
+            break;
+        }
+        w = w->parentWidget();
+    }
+    for (w = m_configWidget ? m_configWidget : m_ui.configWidget; w; w = w->parentWidget()) {
+        if (w->layout())
+            w->layout()->activate();
+    }
 }
 
 void FindToolWindow::acceptAndGetParameters(QString *term, IFindFilter **filter)
@@ -222,9 +236,9 @@ void FindToolWindow::replace()
 
 void FindToolWindow::writeSettings()
 {
-    QSettings *settings = Core::ICore::instance()->settings();
-    settings->beginGroup("Find");
-    settings->setValue("CurrentFilter", m_currentFilter ? m_currentFilter->id() : 0);
+    QSettings *settings = Core::ICore::settings();
+    settings->beginGroup(QLatin1String("Find"));
+    settings->setValue(QLatin1String("CurrentFilter"), m_currentFilter ?  m_currentFilter->id() : QString());
     foreach (IFindFilter *filter, m_filters)
         filter->writeSettings(settings);
     settings->endGroup();
@@ -232,9 +246,9 @@ void FindToolWindow::writeSettings()
 
 void FindToolWindow::readSettings()
 {
-    QSettings *settings = Core::ICore::instance()->settings();
-    settings->beginGroup("Find");
-    const QString currentFilter = settings->value("CurrentFilter").toString();
+    QSettings *settings = Core::ICore::settings();
+    settings->beginGroup(QLatin1String("Find"));
+    const QString currentFilter = settings->value(QLatin1String("CurrentFilter")).toString();
     for (int i = 0; i < m_filters.size(); ++i) {
         IFindFilter *filter = m_filters.at(i);
         filter->readSettings(settings);
